@@ -2,6 +2,7 @@
 
 See test_run for details."""
 
+
 import sys
 
 # We want to ensure that while objects remain available for immediate access,
@@ -10,10 +11,11 @@ import sys
 class C(object):
     def __init__(self,name):
         self.name = name
+        self.p = print
         self.flush_stdout = sys.stdout.flush
         
     def __del__(self):
-        print 'tclass.py: deleting object:',self.name
+        self.p('tclass.py: deleting object:',self.name)
         self.flush_stdout()
 
 try:
@@ -24,9 +26,9 @@ else:
     if name.startswith('C'):
         c = C(name)
 
-#print >> sys.stderr, "ARGV:", sys.argv  # dbg
+# print("ARGV:", sys.argv, file=sys.stderr)  # dbg
 
 # This next print statement is NOT debugging, we're making the check on a
 # completely separate process so we verify by capturing stdout:
-print 'ARGV 1-:', sys.argv[1:]
+print('ARGV 1-:', sys.argv[1:])
 sys.stdout.flush()
